package de.ozgcloud.admin.common;

import java.io.IOException;
import java.util.Collection;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;

import de.ozgcloud.common.errorhandling.TechnicalException;

abstract class AbstractLinkedResourceSerializer extends JsonSerializer<Object> implements ContextualSerializer {
	@Override
	public void serialize(Object value, JsonGenerator gen, SerializerProvider serializers) throws IOException {

		if (value instanceof Collection) {
			gen.writeStartArray();
			((Collection<?>) value).forEach(val -> writeObject(gen, buildLink(val)));
			gen.writeEndArray();
		} else {
			writeObject(gen, buildLink(value));
		}
	}

	abstract String buildLink(Object id);

	abstract IdExtractor<Object> getExtractor();

	void writeObject(JsonGenerator gen, Object value) {
		try {
			gen.writeObject(value);
		} catch (IOException e) {
			throw new TechnicalException("Error writing String to json", e);
		}
	}
}
