package de.ozgcloud.admin.common;

import java.nio.charset.StandardCharsets;
import java.util.Collection;
import java.util.Optional;
import java.util.stream.StreamSupport;

import io.grpc.Metadata;
import io.grpc.Metadata.Key;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class GrpcUtil {

	public static final String ZUFI_MANAGER_GRPC_CLIENT = "zufi-manager";

	public static final String SERVICE_KEY = "GRPC_SERVICE";

	public static Key<String> keyOfString(String key) {
		return Key.of(key, Metadata.ASCII_STRING_MARSHALLER);
	}

	public static Key<byte[]> createKeyOf(String key) {
		return Key.of(key, Metadata.BINARY_BYTE_MARSHALLER);
	}

	public static String getFromHeaders(String key, Metadata headers) {
		return Optional.ofNullable(headers.get(createKeyOf(key)))
				.map(GrpcUtil::byteToString)
				.orElse(null);
	}

	public static Collection<String> getCollection(String key, Metadata headers) {
		return StreamSupport.stream(headers.getAll(createKeyOf(key)).spliterator(), false)
				.map(GrpcUtil::byteToString)
				.toList();
	}

	private static String byteToString(byte[] bytes) {
		return new String(bytes, StandardCharsets.UTF_8);
	}
}