package de.ozgcloud.admin.common;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.lang.reflect.InvocationTargetException;

import org.apache.commons.lang3.reflect.ConstructorUtils;

import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;

import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.NoArgsConstructor;

@NoArgsConstructor
public class LinkedResourceSerializer extends AbstractLinkedResourceSerializer {

	private LinkedResource annotation;

	private LinkedResourceSerializer(LinkedResource annotation) {
		this.annotation = annotation;
	}

	@Override
	public JsonSerializer<?> createContextual(SerializerProvider prov, BeanProperty property) {
		return new LinkedResourceSerializer(property.getAnnotation(LinkedResource.class));
	}

	@Override
	String buildLink(Object id) {
		return linkTo(annotation.controllerClass()).slash(getExtractor().extractId(id)).toString();
	}

	@Override
	IdExtractor<Object> getExtractor() {
		try {
			return ConstructorUtils.invokeConstructor(annotation.extractor());
		} catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException | InstantiationException e) {
			throw new TechnicalException("Error instanciating IdExtractor", e);
		}
	}
}
