package de.ozgcloud.admin.keycloak;

import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.keycloak.representations.idm.GroupRepresentation;
import org.mapstruct.AfterMapping;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.NullValueCheckStrategy;
import org.springframework.beans.factory.annotation.Autowired;

import lombok.extern.log4j.Log4j2;

@Mapper(nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
@Log4j2
abstract class GroupMapper {

	@Autowired
	private KeycloakApiProperties keycloakApiProperties;

	public abstract List<Group> fromGroupRepresentations(List<GroupRepresentation> groupRepresentations);

	@Mapping(target = "organisationsEinheitId", source = "attributes")
	public abstract Group fromGroupRepresentation(GroupRepresentation groupRepresentation);

	String getOrganisationsEinheitId(Map<String, List<String>> attributes) {
		if (attributes == null) {
			return null;
		}
		var values = attributes.get(keycloakApiProperties.getOrganisationsEinheitIdKey());
		if (values == null) {
			return null;
		}
		if (values.size() > 1 && values.stream().distinct().count() > 1) {
			throw new GroupRepresentationMappingException("Group contains multiple values for organisationsEinheitId: %s".formatted(values));
		}
		return values.getFirst();
	}

	@AfterMapping
	protected void deleteGroupsWithoutOrganisationsEinheitId(@MappingTarget List<Group> groups) {
		groups.removeIf(this::isMissingOrganisationsEinheitId);
	}

	protected boolean isMissingOrganisationsEinheitId(Group group) {
		return StringUtils.isBlank(group.getOrganisationsEinheitId());
	}

	@Mapping(target = "attributes", expression = "java(buildGroupAttributes(groupToAdd))")
	public abstract GroupRepresentation toGroupRepresentation(AddGroupData groupToAdd);

	Map<String, List<String>> buildGroupAttributes(AddGroupData groupToAdd) {
		var organisationsEinheitId = groupToAdd.getOrganisationsEinheitId();
		return StringUtils.isNotBlank(organisationsEinheitId) ?
				Map.of(keycloakApiProperties.getOrganisationsEinheitIdKey(), List.of(organisationsEinheitId)) :
				Map.of();
	}
}
