package de.ozgcloud.admin.organisationseinheit;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.stream.StreamSupport;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.mediatype.hal.HalModelBuilder;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.ozgcloud.admin.common.CollectionModelBuilder;

@Component
class OrganisationsEinheitModelAssembler
		implements RepresentationModelAssembler<OrganisationsEinheit, EntityModel<OrganisationsEinheit>> {

	static final String REL_CHILD_ORGANISATIONS_EINHEITEN = "childList";

	static final String REL_EDIT = "edit";

	@Override
	public EntityModel<OrganisationsEinheit> toModel(OrganisationsEinheit organisationsEinheit) {
		var halModelBuilder = HalModelBuilder.halModelOf(organisationsEinheit);

		embedChildOrganisationsEinheiten(organisationsEinheit, halModelBuilder);

		return halModelBuilder
				.<EntityModel<OrganisationsEinheit>>build()
				.add(linkTo(methodOn(OrganisationsEinheitController.class).getById(organisationsEinheit.getId())).withSelfRel())
				.add(linkTo(methodOn(OrganisationsEinheitController.class).editOrganisationsEinheit(organisationsEinheit.getId(), null)).withRel(REL_EDIT));
	}

	void embedChildOrganisationsEinheiten(OrganisationsEinheit organisationsEinheit, HalModelBuilder halModelBuilder) {
		if (CollectionUtils.isNotEmpty(organisationsEinheit.getChildren())) {
			var childrenModel = toChildrenCollectionModel(organisationsEinheit.getId(), organisationsEinheit.getChildren()).getContent();
			halModelBuilder.embed(childrenModel, LinkRelation.of(REL_CHILD_ORGANISATIONS_EINHEITEN))
					.link(linkTo(methodOn(OrganisationsEinheitController.class).getChildren(organisationsEinheit.getId())).withRel(
							REL_CHILD_ORGANISATIONS_EINHEITEN));
		}
	}

	@Override
	public CollectionModel<EntityModel<OrganisationsEinheit>> toCollectionModel(
			Iterable<? extends OrganisationsEinheit> organisationsEinheiten) {
		var models = StreamSupport.stream(organisationsEinheiten.spliterator(), false).map(this::toModel);
		return CollectionModelBuilder.fromEntities(models)
				.addLink(linkTo(methodOn(OrganisationsEinheitController.class).getAll()).withSelfRel())
				.buildModel();
	}

	public CollectionModel<EntityModel<OrganisationsEinheit>> toChildrenCollectionModel(String parentId,
			Iterable<? extends OrganisationsEinheit> organisationsEinheiten) {
		var models = StreamSupport.stream(organisationsEinheiten.spliterator(), false).map(this::toModel);
		return CollectionModelBuilder.fromEntities(models)
				.addLink(linkTo(methodOn(OrganisationsEinheitController.class).getChildren(parentId)).withSelfRel())
				.buildModel();
	}
}
