package de.ozgcloud.admin.setting;

import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.stereotype.Service;

import de.ozgcloud.admin.organisationseinheit.OrganisationsEinheitSettings;
import de.ozgcloud.admin.setting.postfach.PostfachSettingBody;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
class SettingService {
	private static final String POSTFACH_SETTING_ITEM_NAME = "Postfach";

	private final OrganisationsEinheitSettingsRepository organisationsEinheitSettingsRepository;
	private final SettingRepository repository;

	private final SettingMapper mapper;

	public AlfaSettingDTO getAlfaSettingDTO() {
		var postfachData = getSettingWithPostfachFromDb();
		Map<String, OrganisationsEinheitSettings> settings = getOrganisationsEinheitSettings();

		return AlfaSettingDTO.builder()
				.signatur(postfachData.getSignatur())
				.organisationsEinheitSettings(settings)
				.build();
	}

	public VorgangManagerSettingDTO getVorgangManagerSettingDTO() {
		var postfachData = getSettingWithPostfachFromDb();
		Map<String, OrganisationsEinheitSettings> settings = getOrganisationsEinheitSettings();

		return VorgangManagerSettingDTO.builder()
				.absender(mapper.mapAbsenderToAbsenderVorgangManager(postfachData.getAbsender()))
				.signatur(postfachData.getSignatur())
				.organisationsEinheitSettings(settings)
				.build();
	}

	PostfachSettingBody getSettingWithPostfachFromDb() {
		var postfach = repository.findOneByName(POSTFACH_SETTING_ITEM_NAME);
		return postfach.isPresent() ? (PostfachSettingBody) postfach.get().getSettingBody() : PostfachSettingBody.builder().build();
	}

	Map<String, OrganisationsEinheitSettings> getOrganisationsEinheitSettings() {
		return organisationsEinheitSettingsRepository.getSettings().collect(Collectors.toMap(
				OrganisationsEinheitWithSettings::organisationsEinheitId,
				OrganisationsEinheitWithSettings::settings));
	}
}
