package de.ozgcloud.admin.migration;

import static de.ozgcloud.admin.migration.M001_CreateEmptyPostfachIfMissing.*;
import static org.assertj.core.api.Assertions.*;

import java.util.List;

import org.bson.Document;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;

import de.ozgcloud.common.test.DataITCase;

@DataITCase
class M001_CreateEmptyPostfachIfMissingITCase {
	private final M001_CreateEmptyPostfachIfMissing changeUnit = new M001_CreateEmptyPostfachIfMissing();

	private static final Document POSTFACH = MigrationTestFactory.createDummyPostfach();

	@Autowired
	private MongoTemplate template;

	@DisplayName("Do migration")
	@Nested
	class TestDoMigration {
		@BeforeEach
		public void init() {
			template.dropCollection(SETTINGS_COLLECTION);
		}

		@Test
		void shouldAddPostfachIfEmpty() {
			changeUnit.doMigration(template);

			List<Document> settings = findAllSettings();

			assertThat(settings).usingRecursiveFieldByFieldElementComparatorIgnoringFields("_id")
					.containsExactly(MigrationTestFactory.createEmptyPostfach());
		}

		@Test
		void shouldAddPostfachIfMissingInCollection() {
			var settingItem = MigrationTestFactory.createSettingsItem("SomeType", new Document());
			template.save(settingItem, SETTINGS_COLLECTION);

			changeUnit.doMigration(template);

			List<Document> settings = findAllSettings();
			assertThat(settings)
					.extracting(TYPE_NAME_KEY)
					.containsExactly("SomeType", MigrationTestFactory.ITEM_TYPE_VALUE_POSTFACH);
		}

		@Test
		void shouldKeepExistingPostfach() {
			template.save(POSTFACH, SETTINGS_COLLECTION);

			changeUnit.doMigration(template);

			List<Document> settings = findAllSettings();
			assertThat(settings).containsExactly(POSTFACH);
		}

		private List<Document> findAllSettings() {
			return template.findAll(Document.class, SETTINGS_COLLECTION);
		}

	}

	static class MigrationTestFactory {

		public static final String ITEM_TYPE_KEY = "name";
		public static final String ITEM_SETTINGS_KEY = "settings";
		public static final String ITEM_ID = "_id";
		public static final String ITEM_TYPE_VALUE_POSTFACH = "Postfach";
		public static final String POSTFACH_ABSENDER_KEY = "absender";

		static Document createDummyPostfach() {
			var postfach = new Document();
			postfach.put(POSTFACH_ABSENDER_KEY, "Some Value");
			return createSettingsItem(ITEM_TYPE_VALUE_POSTFACH, postfach);
		}

		static Document createEmptyPostfach() {
			var postfach = new Document();
			postfach.put(ITEM_TYPE_KEY, ITEM_TYPE_VALUE_POSTFACH);
			postfach.put(ITEM_ID, null);
			return postfach;
		}

		static Document createSettingsItem(String itemType, Document itemValue) {
			var settingsItem = new Document();
			settingsItem.put(ITEM_TYPE_KEY, itemType);
			settingsItem.put(ITEM_SETTINGS_KEY, itemValue);
			return settingsItem;
		}

	}
}
