package de.ozgcloud.admin.organisationseinheit;

import static org.assertj.core.api.Assertions.*;

import java.util.Collections;
import java.util.Objects;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.admin.keycloak.AddGroupData;

class OrganisationsEinheitMapperTest {

	private final OrganisationsEinheitMapper mapper = Mappers.getMapper(OrganisationsEinheitMapper.class);

	@Nested
	class TestFromGrpc {

		@Test
		void shouldMap() {
			var mapped = mapper.fromGrpc(GrpcOrganisationsEinheitTestFactory.create());

			assertThat(mapped)
					.matches(organisationsEinheit -> Objects.isNull(organisationsEinheit.getSettings().getSignatur()))
					.extracting(
							OrganisationsEinheit::getName,
							OrganisationsEinheit::getOrganisationsEinheitId,
							OrganisationsEinheit::getZufiId,
							OrganisationsEinheit::getParentId,
							OrganisationsEinheit::getSyncResult,
							OrganisationsEinheit::getChildren
					).containsExactly(
							OrganisationsEinheitTestFactory.NAME,
							OrganisationsEinheitTestFactory.ORGANISATIONS_EINHEIT_ID,
							OrganisationsEinheitTestFactory.ZUFI_ID,
							null,
							null,
							Collections.emptyList()
					);
		}
	}

	@Nested
	class TestToAddGroupData {

		@Test
		void shouldMap() {
			var mapped = mapper.toAddGroupData(OrganisationsEinheitTestFactory.create());

			assertThat(mapped).extracting(
					AddGroupData::getName,
					AddGroupData::getOrganisationsEinheitId
			).containsExactly(
					OrganisationsEinheitTestFactory.NAME,
					OrganisationsEinheitTestFactory.ORGANISATIONS_EINHEIT_ID
			);
		}
	}
}