package de.ozgcloud.admin.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcGetByOrganisationsEinheitIdRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcGetByOrganisationsEinheitIdResponse;
import de.ozgcloud.zufi.grpc.organisationseinheit.OrganisationsEinheitServiceGrpc.OrganisationsEinheitServiceBlockingStub;

class OrganisationsEinheitRemoteServiceTest {

	@Spy
	@InjectMocks
	private OrganisationsEinheitRemoteService service;

	@Mock
	private OrganisationsEinheitServiceBlockingStub serviceStub;

	@Mock
	private OrganisationsEinheitMapper mapper;

	@Nested
	class TestGetByOrganisationsEinheitId {

		private final GrpcGetByOrganisationsEinheitIdRequest request = GrpcGetByOrganisationsEinheitIdRequestTestFactory.create();
		private final GrpcGetByOrganisationsEinheitIdResponse response = GrpcGetByOrganisationsEinheitIdResponseTestFactory.create();
		private final OrganisationsEinheit organisationsEinheit = OrganisationsEinheitTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(request).when(service).buildGetByOrganisationsEinheitIdRequest(OrganisationsEinheitTestFactory.ORGANISATIONS_EINHEIT_ID);
			when(serviceStub.getByOrganisationsEinheitId(request)).thenReturn(response);
			when(mapper.fromGrpc(GrpcGetByOrganisationsEinheitIdResponseTestFactory.GRPC_ORGANISATIONS_EINHEIT)).thenReturn(organisationsEinheit);
		}

		@Test
		void shouldBuildRequest() {
			service.getByOrganisationsEinheitId(OrganisationsEinheitTestFactory.ORGANISATIONS_EINHEIT_ID);

			verify(service).buildGetByOrganisationsEinheitIdRequest(OrganisationsEinheitTestFactory.ORGANISATIONS_EINHEIT_ID);
		}

		@Test
		void shouldCallServiceStub() {
			service.getByOrganisationsEinheitId(OrganisationsEinheitTestFactory.ORGANISATIONS_EINHEIT_ID);

			verify(serviceStub).getByOrganisationsEinheitId(request);
		}

		@Test
		void shouldMapResponse() {
			service.getByOrganisationsEinheitId(OrganisationsEinheitTestFactory.ORGANISATIONS_EINHEIT_ID);

			verify(mapper).fromGrpc(GrpcGetByOrganisationsEinheitIdResponseTestFactory.GRPC_ORGANISATIONS_EINHEIT);
		}

		@Test
		void shouldReturnOrganisationsEinheiten() {
			var organisationsEinheiten = service.getByOrganisationsEinheitId(OrganisationsEinheitTestFactory.ORGANISATIONS_EINHEIT_ID);

			assertThat(organisationsEinheiten).containsExactly(organisationsEinheit);

		}
	}

	@Nested
	class TestBuildGetByOrganisationsEinheitIdRequest {

		@Test
		void shouldBuild() {
			var request = service.buildGetByOrganisationsEinheitIdRequest(OrganisationsEinheitTestFactory.ID);

			assertThat(request.getOrganisationsEinheitId()).isEqualTo(OrganisationsEinheitTestFactory.ID);
		}
	}

}