package de.ozgcloud.admin.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.Instant;

import org.assertj.core.data.*;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

class SyncSchedulerTest {

	@InjectMocks
	private SyncScheduler scheduler;

	@Mock
	private SyncService syncService;

	@Nested
	class TestSyncOrganisationsEinheitenWithKeycloak {

		@Captor
		private ArgumentCaptor<Long> syncTimestampArgumentCaptor;

		@Test
		void shouldSyncOrganisationsEinheitenFromKeycloak() {
			callService();

			verify(syncService).syncOrganisationsEinheitenFromKeycloak(syncTimestampArgumentCaptor.capture());
			assertThatSyncTimestampIsCloseToNow();
		}

		@Test
		void shouldSyncAddedOrganisationsEinheiten() {
			callService();

			verify(syncService).syncAddedOrganisationsEinheiten(syncTimestampArgumentCaptor.capture());
			assertThatSyncTimestampIsCloseToNow();
		}

		private void assertThatSyncTimestampIsCloseToNow() {
			assertThat(syncTimestampArgumentCaptor.getValue()).isCloseTo(Instant.now().toEpochMilli(), Offset.offset(5000L));
		}

		private void callService() {
			scheduler.syncOrganisationsEinheitenWithKeycloak();
		}
	}

}